@@ -58,8 +58,7 @@ compute.gene.cdf <- function(expr, Gaussk=TRUE, kernel=TRUE,
                       verbose)
             gene.cdf <- t(matrix(A, n.test.samples, n.genes))
         } else
-            cli_abort(c("x"=sprintf("Matrix class %s cannot be handled yet.",
-                                    class(expr))))
+            cli_abort("x"=sprintf("Matrix class %s cannot be handled yet.", class(expr)))
     } else { ## direct ECDF estimation
         if (is(expr, "dgCMatrix")) {
             if (sparse)
@@ -89,8 +88,8 @@ compute.gene.cdf <- function(expr, Gaussk=TRUE, kernel=TRUE,
             else
                 gene.cdf <- .ecdfvals_dense_to_dense(expr, verbose)
         } else
-            cli_abort(c("x"=sprintf("Input container class %s cannot be handled yet.",
-                                    class(expr))))
+            cli_abort("x"=sprintf("Input container class %s cannot be handled yet.",
+                         class(expr)))
     }
 
     return(gene.cdf)	
@@ -222,19 +221,8 @@ zorder_rankstat <- function(z, p) {
 #' @param verbose Gives information about each calculation step. Default: `TRUE`.
 #'
 #' @param BPPARAM An object of class `BiocParallelParam` specifying parameters
-#' related to the parallel execution of some of the tasks and calculations
-#' within this function.
-#'
-#' @param maxmem A vector of length 1 either specifying a number in bytes, or
-#' a character string with either the word `auto` (default), or a number
-#' followed by a suffix indicating kilobytes (K), megabytes (M), gigabytes (G)
-#' or terabytes (T), which GSVA will use to attempt bounding the maximum amount
-#' of main memory used across all threads of execution to that given quantity.
-#' By default `maxmem="auto"`, indicating that the maximum memory will be the
-#' 90% of the total main memory, as calculated by [`Sys.meminfo()`][memuse::Sys.meminfo].
-#' To avoid setting any bound on the maximum memory, please use `maxmem=Inf`.
-#' Note that the amount of main memory used in an R session or script may depend
-#' on other commands and packages used in that same session or script.
+#'   related to the parallel execution of some of the tasks and calculations
+#'   within this function.
 #'
 #' @return In the case of the `gsvaRanks()` method, an object of class
 #' [`gsvaRanksParam-class`].
@@ -309,8 +297,7 @@ zorder_rankstat <- function(z, p) {
 setMethod("gsvaRanks", signature(param="gsvaParam"),
           function(param,
                    verbose=TRUE,
-                   BPPARAM=SerialParam(progressbar=verbose),
-                   maxmem="auto")
+                   BPPARAM=SerialParam(progressbar=verbose))
           {
               if (verbose && gsva_global$show_start_and_end_messages) {
                   cli_alert_info(sprintf("GSVA version %s",
@@ -319,25 +306,25 @@ setMethod("gsvaRanks", signature(param="gsvaParam"),
 
               exprData <- get_exprData(param)
               dataMatrix <- unwrapData(exprData, get_assay(param))
-              maxmem <- .check_maxmem(param, maxmem, verbose)
-              ondisk <- .check_ondisk(param, maxmem, verbose)
 
-              if (is(dataMatrix, "DelayedMatrix") && ondisk == "no") {
+              if (is(dataMatrix, "DelayedMatrix") && get_ondisk(param) == "no") {
                   if (verbose)
                       cli_alert_info("Loading input expression data into main memory")
                   if (is_sparse(dataMatrix)) {
-                      dataMatrix <- as(dataMatrix, "SVT_SparseArray")
+                      if (nzcount(param) < .Machine$integer.max)
+                          dataMatrix <- as(dataMatrix, "dgCMatrix")
+                      else
+                          dataMatrix <- as(dataMatrix, "SVT_SparseArray")
                   } else
                       dataMatrix <- as.matrix(dataMatrix)
               }
 
-              filtDataMatrix <- dataMatrix
+              filteredDataMatrix <- dataMatrix
               if (get_filterRows(param))
-                  filtDataMatrix <- .filterGenes(dataMatrix, anyNA(param),
-                                                 removeConstant=TRUE,
-                                                 removeNzConstant=TRUE,
-                                                 verbose, BPPARAM=BPPARAM,
-                                                 maxmem=maxmem)
+                  filteredDataMatrix <- .filterGenes(dataMatrix,
+                                                     removeConstant=TRUE,
+                                                     removeNzConstant=TRUE,
+                                                     verbose, BPPARAM=BPPARAM)
               else if (verbose)
                   cli_alert_warning("Skipping filtering of constant rows (filterRows=FALSE)")
               
@@ -345,18 +332,17 @@ setMethod("gsvaRanks", signature(param="gsvaParam"),
                   cli_alert_info(sprintf("Calculating GSVA ranks"))
 
               kcdfminssize <-get_kcdfNoneMinSampleSize(param)
-              gsvarnks <- .compute_gsva_ranks(expr=filtDataMatrix,
+              gsvarnks <- .compute_gsva_ranks(expr=filteredDataMatrix,
                                               kcdf=get_kcdf(param),
                                               kcdf.min.ssize=kcdfminssize,
                                               sparse=get_sparse(param),
                                               any_na=anyNA(param),
                                               na_use=get_NAuse(param),
                                               verbose=verbose,
-                                              BPPARAM=BPPARAM,
-                                              maxmem=maxmem)
+                                              BPPARAM=BPPARAM)
 
-              rownames(gsvarnks) <- rownames(filtDataMatrix)
-              colnames(gsvarnks) <- colnames(filtDataMatrix)
+              rownames(gsvarnks) <- rownames(filteredDataMatrix)
+              colnames(gsvarnks) <- colnames(filteredDataMatrix)
 
               rnkcontainer <- wrapData(get_exprData(param), gsvarnks)
               rval <- new("gsvaRanksParam",
@@ -462,8 +448,8 @@ setMethod("gsvaRanks", signature(param="gsvaParam"),
 #' @exportMethod gsvaScores
 setMethod("gsvaScores", signature(param="gsvaRanksParam"),
           function(param, verbose=TRUE,
-                   BPPARAM=SerialParam(progressbar=verbose),
-                   maxmem="auto") {
+                   BPPARAM=SerialParam(progressbar=verbose))
+          {
               if (verbose && gsva_global$show_start_and_end_messages) {
                   cli_alert_info(sprintf("GSVA version %s",
                                          packageDescription("GSVA")[["Version"]]))
@@ -471,14 +457,13 @@ setMethod("gsvaScores", signature(param="gsvaRanksParam"),
 
               ## assuming rows in the rank data have been already filtered
               exprData <- get_exprData(param)
-              filtDataMatrix <- unwrapData(exprData, get_assay(param))
-
-              filtMappedGeneSets <- .filterAndMapGeneSets(param=param,
-                                           filteredDataMatrix=filtDataMatrix,
-                                           verbose=verbose)
+              filteredDataMatrix <- unwrapData(exprData, get_assay(param))
+              filteredMappedGeneSets <- .filterAndMapGeneSets(param=param,
+                                                              filteredDataMatrix=filteredDataMatrix,
+                                                              verbose=verbose)
 
               sparse <- get_sparse(param)
-              if (sparse && !is_sparse(filtDataMatrix))
+              if (sparse && !is_sparse(filteredDataMatrix))
                   sparse <- FALSE
 
               if (verbose) {
@@ -488,23 +473,8 @@ setMethod("gsvaScores", signature(param="gsvaRanksParam"),
                     cli_alert_info("GSVA dense (classical) algorithm")
               }
 
-              maxmem <- .check_maxmem(param, maxmem, verbose)
-              ondisk <- .check_ondisk(param, maxmem, verbose)
-
-              if (is(filtDataMatrix, "DelayedMatrix") && ondisk == "no") {
-                  if (verbose)
-                      cli_alert_info("Loading input expression data into main memory")
-                  if (is_sparse(filtDataMatrix)) {
-                      if (nzcount(param) < .Machine$integer.max)
-                          filtDataMatrix <- as(filtDataMatrix, "dgCMatrix")
-                      else
-                          filtDataMatrix <- as(filtDataMatrix, "SVT_SparseArray")
-                  } else
-                      filtDataMatrix <- as.matrix(filtDataMatrix)
-              }
-
-              if (bpnworkers(BPPARAM) > 1 && nrow(filtDataMatrix) > 100 &&
-                  ncol(filtDataMatrix) > 100) {
+              if (bpnworkers(BPPARAM) > 1 && nrow(filteredDataMatrix) > 100 &&
+                  ncol(filteredDataMatrix) > 100) {
                   if (verbose) {
                       msg <- sprintf("Calculating GSVA scores with %d cores",
                                      as.integer(bpnworkers(BPPARAM)))
@@ -516,37 +486,23 @@ setMethod("gsvaScores", signature(param="gsvaRanksParam"),
                   BPPARAM <- NULL
               }
 
-              ondisk <- FALSE
-              esreqmem <- as.numeric(length(filtMappedGeneSets)) *
-                          as.numeric(ncol(filtDataMatrix)) * 8 ## 8 bytes per double
-              if (esreqmem > maxmem) {
-                cli_alert_warning("The resulting matrix of enrichment scores will not fit")
-                cli_alert_warning("in the given maximum main memory size, the returned")
-                cli_alert_warning("object will use an on-disk data structure")
-                ondisk <- TRUE
-              }
-
-              gsva_es <- .processMatrixCols(filtDataMatrix,
+              gsva_es <- .processMatrixCols(filteredDataMatrix,
                                             FUN=.compute_gsva_scores,
-                                            geneSetsIdx=filtMappedGeneSets,
+                                            geneSetsIdx=filteredMappedGeneSets,
                                             tau=get_tau(param),
                                             maxDiff=get_maxDiff(param),
                                             absRanking=get_absRanking(param),
                                             sparse=sparse, any_na=anyNA(param),
                                             na_use=get_NAuse(param),
                                             minSize=get_minSize(param),
-                                            ondisk=ondisk, verbose=verbose,
-                                            minparrows=100, minparcols=100,
-                                            BPPARAM=BPPARAM,
-                                            maxmem=ceiling(maxmem/100)) ## use
-                                            ## of memory increases here about
-                                            ## 10-fold over block size memory
+                                            verbose=verbose, minparrows=100,
+                                            minparcols=100, BPPARAM=BPPARAM)
 
-              rownames(gsva_es) <- names(filtMappedGeneSets)
-              colnames(gsva_es) <- colnames(filtDataMatrix)
+              rownames(gsva_es) <- names(filteredMappedGeneSets)
+              colnames(gsva_es) <- colnames(filteredDataMatrix)
 
-              gs <- .geneSetsIndices2Names(indices=filtMappedGeneSets,
-                                           names=rownames(filtDataMatrix))
+              gs <- .geneSetsIndices2Names(indices=filteredMappedGeneSets,
+                                           names=rownames(filteredDataMatrix))
               rval <- wrapData(get_exprData(param), gsva_es, gs)
 
               if (verbose && gsva_global$show_start_and_end_messages)
@@ -665,16 +621,15 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
               minsize <- get_minSize(param)
 
               exprData <- get_exprData(param)
-              filtDataMatrix <- unwrapData(exprData, get_assay(param))
+              filteredDataMatrix <- unwrapData(exprData, get_assay(param))
 
               ## no need for verbosity when mapping a single gene set
-              filtMappedGeneSets <- .filterAndMapGeneSets(param,
-                                           wgset=geneSet,
-                                           filteredDataMatrix=filtDataMatrix,
-                                           verbose=FALSE)
+              filteredMappedGeneSets <- .filterAndMapGeneSets(param, wgset=geneSet,
+                                                              filteredDataMatrix,
+                                                              verbose=FALSE)
 
-              geneSetIdx <- filtMappedGeneSets[[1]]
-              edata <- .gsva_enrichment_data(R=filtDataMatrix,
+              geneSetIdx <- filteredMappedGeneSets[[1]]
+              edata <- .gsva_enrichment_data(R=filteredDataMatrix,
                                              column=column,
                                              geneSetIdx=geneSetIdx,
                                              maxDiff=maxDiff,
@@ -755,7 +710,7 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
 #' @importFrom sparseMatrixStats colRanks
 .compute_gsva_ranks <- function(expr, kcdf, kcdf.min.ssize,
                                 sparse, any_na, na_use, verbose,
-                                BPPARAM=NULL, maxmem=Inf) {
+                                BPPARAM=NULL) {
 
     kcdfparam <- .parse_kcdf_param(expr, kcdf, kcdf.min.ssize, sparse, verbose)
     kernel <- kcdfparam$kernel
@@ -776,7 +731,7 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
     Z <- .processMatrixRows(expr, FUN=compute.gene.cdf, Gaussk=Gaussk,
                             kernel=kernel, sparse=sparse, any_na=any_na,
                             na_use=na_use, verbose=verbose, minparrows=100,
-                            minparcols=100, BPPARAM=BPPARAM, maxmem=maxmem)
+                            minparcols=100, BPPARAM=BPPARAM)
 
     if (!is.null(BPPARAM) && verbose) {
         msg <- sprintf("Calculating column ranks with %d cores",
@@ -789,7 +744,7 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
     ## from 'order()' based on ranks
     R <- .processMatrixCols(Z, FUN=compute.col.ranks, ties.method="last",
                             verbose=verbose, minparrows=100, minparcols=100,
-                            BPPARAM=BPPARAM, maxmem=maxmem)
+                            BPPARAM=BPPARAM)
 
     return(R)
 }
@@ -981,10 +936,9 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
 
     if (all(na_mask))
         return(list(dos=matrix(NA_integer_, nrow(r), ncol(r)),
-                    srs=matrix(NA_real_, nrow(r), ncol(r))))
+                    srs=matrix(NA, nrow(r), ncol(r))))
 
     n_nas <- colSums(na_mask)
-    mode(n_nas) <- "integer"
     mask <- !na_mask & mask
     p <- nrow(r)
     r_dense <- as.matrix(r)
@@ -1027,14 +981,12 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
         return(list(dos=rep(NA, length(r)), srs=rep(NA, length(r))))
 
     n_nas <- sum(na_mask)
-    mode(n_nas) <- "integer"
     mask <- !na_mask & r == 0
     p <- length(r)
     r_dense <- as.integer(r)          ## assume ranks are integer
 
     if (any(mask)) {                  ## sparse ranks into dense ranks
         nzs <- sum(mask)
-        mode(nzs) <- "integer"
         r_dense[!mask] <- r_dense[!mask] + nzs ## shift ranks of nonzero values
         r_dense[mask] <- seq.int(nzs)          ## zeros get increasing ranks
     }
@@ -1057,8 +1009,8 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
 #' @importFrom BiocParallel bpnworkers
 #' @importFrom S4Arrays is_sparse refdim
 .compute_gsva_scores <- function(R, geneSetsIdx, tau, maxDiff, absRanking,
-                                 sparse, any_na, na_use, minSize, ondisk,
-                                 verbose) {
+                                 sparse, any_na, na_use, minSize,
+                                 grid=NULL, grid_es=NULL, verbose) {
     p <- nrow(R)
     n <- ncol(R)
     es <- NULL
@@ -1068,23 +1020,32 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
     wna_env <- new.env()
     assign("w", FALSE, envir=wna_env)
     es <- NULL
-    if (is(R, "DelayedMatrix") || ondisk) {
+    if (is(R, "DelayedMatrix")) {
         sink <- HDF5RealizationSink(c(length(geneSetsIdx), ncol(R)),
                                     as.sparse=FALSE) ## GSVA scores are dense
-        grid <- DummyArrayGrid(dim(R))
-        grid_es <- DummyArrayGrid(dim(sink))
+        if (is.null(grid))
+            grid <- DummyArrayGrid(dim(R))
+        if (is.null(grid_es))
+            grid_es <- DummyArrayGrid(dim(sink))
 
         if (length(grid) != length(grid_es) ||
             refdim(grid)[2] != refdim(grid_es)[2] ||
             dim(grid)[2] != dim(grid_es)[2]) {
-            cli_abort(c("x"="Grid column blocks for ranks should match grid column blocks for enrichment scores"))
+            cli_abort("x"="Grid column blocks for ranks should match grid column blocks for enrichment scores")
         }
 
         ## avp - ArrayViewport for reaching the (possibly sparse) rank matrix
         ## avp_es - ArrayViewport for writing the enrichment dense scores matrix
         colScores_byBlock <- function(avp, avp_es, sink) {
             block <- read_block(R, avp)
-            block <- .gsva_score_genesets(block, geneSetsIdx, sparse,
+            rnkstats <- NULL
+            if (any_na)
+                rnkstats <- .ranks2stats_nas_block(block, sparse)
+            else
+                rnkstats <- .ranks2stats_block(block, sparse)
+            block <- .gsva_score_genesets(NULL, geneSetsIdx,
+                                          decOrdStat=rnkstats$dos,
+                                          symRnkStat=rnkstats$srs,
                                           maxDiff, absRanking, tau,
                                           any_na, na_use, minSize,
                                           wna_env, verbose=verbose)
@@ -1098,9 +1059,17 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
         es <- as(sink, "DelayedArray")
 
     } else {
-        es <- .gsva_score_genesets(R, geneSetsIdx, sparse,
-                                   maxDiff, absRanking, tau, any_na,
-                                   na_use, minSize, wna_env, verbose=verbose)
+        rnkstats <- NULL
+         if (any_na)
+             rnkstats <- .ranks2stats_nas_block(R, sparse)
+         else
+             rnkstats <- .ranks2stats_block(R, sparse)
+
+         es <- .gsva_score_genesets(NULL, geneSetsIdx,
+                                    decOrdStat=rnkstats$dos,
+                                    symRnkStat=rnkstats$srs,
+                                    maxDiff, absRanking, tau, any_na,
+                                    na_use, minSize, wna_env, verbose=verbose)
     }
 
     if (any_na && na_use == "na.rm")
@@ -1281,15 +1250,9 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
 ## functions interfacing C code
 ##
 
-.fetch_row_nzvals <- function(X, i, whimin1=NULL) {
+.fetch_row_nzvals <- function(X, i) {
   stopifnot(is(X, "SVT_SparseArray")) ## QC
-  stopifnot(is.numeric(i)) ## QC
-  if (!is.null(whimin1)) {
-      stopifnot(is.numeric(whimin1)) ## QC
-      whimin1 <- as.integer(whimin1)
-      stopifnot(length(whimin1) == ncol(X)) ## QC
-  }
-  .Call("fetch_row_nzvals_R", X, as.integer(i), whimin1)
+  .Call("fetch_row_nzvals_R", X, as.integer(i))
 }
 
 .ecdfvals_svt_to_dense <- function(X, verbose) {
@@ -1311,8 +1274,7 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
 }
 
 #' @importFrom HDF5Array HDF5RealizationSink
-#' @importFrom S4Arrays DummyArrayGrid
-#' @importFrom DelayedArray seed gridReduce
+#' @importFrom DelayedArray seed DummyArrayGrid gridReduce
 .ecdfvals_sparseh5_to_sparseh5 <- function(X, grid=NULL, verbose=FALSE) {
   stopifnot(is(X, "DelayedMatrix") || is(X, "HDF5Matrix")) ## QC
 
@@ -1332,8 +1294,7 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
 }
 
 #' @importFrom HDF5Array HDF5RealizationSink
-#' @importFrom S4Arrays DummyArrayGrid
-#' @importFrom DelayedArray seed gridReduce
+#' @importFrom DelayedArray seed DummyArrayGrid gridReduce
 .ecdfvals_sparseh5_to_denseh5 <- function(X, grid=NULL, verbose) {
   stopifnot(is(X, "DelayedMatrix") || is(X, "HDF5Matrix")) ## QC
 
@@ -1509,9 +1470,9 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
 }
 
 #' @importFrom cli cli_abort
-.old_gsva_score_genesets <- function(colIdx, geneSetsIdx, decOrdStat, symRnkStat,
-                                     maxDiff, absRanking, tau, any_na, na_use,
-                                     minSize, wna_env, verbose) {
+.gsva_score_genesets <- function(colIdx, geneSetsIdx, decOrdStat, symRnkStat,
+                                 maxDiff, absRanking, tau, any_na, na_use,
+                                 minSize, wna_env, verbose) {
   minSize <- as.integer(minSize)
   stopifnot(is.null(colIdx) || is.integer(colIdx)) ## QC
   stopifnot(is.list(geneSetsIdx)) ## QC
@@ -1538,7 +1499,7 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
     decOrdStat <- decOrdStat[, colIdx, drop=FALSE]
     symRnkStat <- symRnkStat[, colIdx, drop=FALSE]
   }
-  sco <- .Call("old_gsva_score_genesets_R", geneSetsIdx, decOrdStat, symRnkStat,
+  sco <- .Call("gsva_score_genesets_R", geneSetsIdx, decOrdStat, symRnkStat,
                maxDiff, absRanking, as.double(tau), any_na, na_use, minSize,
                verbose)
   if (any_na) {
@@ -1552,46 +1513,6 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
   sco
 }
 
-#' @importFrom cli cli_abort
-.gsva_score_genesets <- function(R, geneSetsIdx, sparse, maxDiff, absRanking,
-                                 tau, any_na, na_use, minSize, wna_env,
-                                 verbose) {
-    minSize <- as.integer(minSize)
-    stopifnot(is.list(geneSetsIdx)) ## QC
-    stopifnot(length(geneSetsIdx) > 0) ## QC
-    stopifnot(is.integer(geneSetsIdx[[1]])) ## QC
-    stopifnot(is.logical(sparse)) ## QC
-    stopifnot(is.logical(maxDiff)) ## QC
-    stopifnot(is.logical(absRanking)) ## QC
-    stopifnot(is.numeric(tau)) ## QC but it still might be an integer!!
-    stopifnot(is.logical(any_na)) ## QC
-    stopifnot(is.character(na_use)) ## QC
-    stopifnot(is.integer(minSize)) ## QC
-    stopifnot(is.logical(verbose)) ## QC
-    na_use <- as.integer(factor(na_use, levels=c("everything", "all.obs",
-                                                 "na.rm")))
-    sco <- .Call("gsva_score_genesets_R", R, geneSetsIdx, sparse, maxDiff,
-                 absRanking, as.double(tau), any_na, na_use, minSize, verbose)
-
-    if (any_na) {
-      if (na_use == 2 && !is.null(attr(sco, "class")))
-          cli_abort(c("x"="Input GSVA ranks have NA values."))
-
-      if (na_use == 3 && !is.null(attr(sco, "class")))
-          assign("w", TRUE, envir=wna_env)
-    }
-
-    sco
-}
-
-.ranks2stats_C <- function(R, j, sparse, anyna) {
-  stopifnot(is.numeric(j)) ## QC
-  stopifnot(is.logical(sparse)) ## QC
-  stopifnot(is.logical(anyna)) ## QC
-  .Call("ranks2stats_R", R, as.integer(j), sparse, anyna)
-}
-
-
 .order_rankstat_sparse_to_dense <- function(X, j) {
   stopifnot(is(X, "CsparseMatrix")) ## QC
   .Call("order_rankstat_sparse_to_dense_R", X, j)
@@ -1613,6 +1534,28 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
 
 ## calculate ranks using an HDF5 backend
 
+## adapted from .define_multiworker_grid() in beachmat/R/colBlockApply.R
+#' @importFrom DelayedArray rowAutoGrid colAutoGrid getAutoBlockLength type
+rowgridsize <- function(X, nworkers) {
+  max.block.length <- getAutoBlockLength(type(X))
+  if (nworkers > 1)
+      max.block.length <- .Machine$integer.max
+  expected.block.length <- max(1, ceiling(nrow(X) / nworkers) * ncol(X))
+  block.length <- min(max.block.length, expected.block.length)
+  rowAutoGrid(X, block.length=block.length)
+}
+
+## adapted from .define_multiworker_grid() in beachmat/R/colBlockApply.R
+#' @importFrom DelayedArray rowAutoGrid colAutoGrid getAutoBlockLength type
+colgridsize <- function(X, nworkers) {
+  max.block.length <- getAutoBlockLength(type(X))
+  if (nworkers > 1)
+      max.block.length <- .Machine$integer.max
+  expected.block.length <- max(1, ceiling(ncol(X) / nworkers) * nrow(X))
+  block.length <- min(max.block.length, expected.block.length)
+  colAutoGrid(X, block.length=block.length)
+}
+
 #' @importFrom MatrixGenerics colRanks
 #' @importFrom BiocParallel SerialParam
 .colRanksHDF5 <- function(X, grid=NULL, ties.method="last") {
@@ -1640,3 +1583,64 @@ setMethod("gsvaEnrichment", signature(param="gsvaRanksParam"),
     res <- as(sink, "DelayedArray")
     res
 }
+
+
+##
+## we may not need anymore these two functions below
+##
+
+ks_test_m <- function(gset_idxs, gene.density, sort.idxs, mx.diff=TRUE,
+                      abs.ranking=FALSE, tau=1, verbose=TRUE){
+	
+	n.genes <- nrow(gene.density)
+	n.samples <- ncol(gene.density)
+	n.geneset <- length(gset_idxs)
+
+	geneset.sample.es <- .Call("ks_matrix_R",
+			                       as.double(gene.density),
+			                       as.integer(sort.idxs),
+			                       n.genes,
+			                       as.integer(gset_idxs),
+			                       n.geneset,
+			                       as.double(tau),
+			                       n.samples,
+			                       as.integer(mx.diff),
+                             as.integer(abs.ranking))
+
+	return(geneset.sample.es)
+}
+
+
+## ks-test in R code - testing only
+ks_test_Rcode <- function(gene.density, gset_idxs, tau=1, make.plot=FALSE){
+	
+	n.genes = length(gene.density)
+	n.gset = length(gset_idxs)
+	
+	sum.gset <- sum(abs(gene.density[gset_idxs])^tau)
+	
+	dec = 1 / (n.genes - n.gset)
+	
+	sort.idxs <- order(gene.density,decreasing=T)
+	offsets <- sort(match(gset_idxs, sort.idxs))
+	
+	last.idx = 0
+	values <- rep(NaN, length(gset_idxs))
+	current = 0
+	for(i in seq_along(offsets)){
+		current = current + abs(gene.density[sort.idxs[offsets[i]]])^tau / sum.gset - dec * (offsets[i]-last.idx-1)
+		
+		values[i] = current
+		last.idx = offsets[i]
+	}
+	check_zero = current - dec * (n.genes-last.idx)
+	#if(check_zero > 10^-15){ 
+	#	stop(paste=c("Expected zero sum for ks:", check_zero))
+	#}
+	if(make.plot){ plot(offsets, values,type="l") } 
+	
+	max.idx = order(abs(values),decreasing=T)[1]
+	mx.value <- values[max.idx]
+	
+	return (mx.value)
+}
