@@ -221,6 +221,12 @@ gsvaParam <- function(exprData, geneSets,
                                     checkNA=checkNA, use=use)
 
     nzc <- .estimate_nzcount(exprData, assay, verbose)
+    if (ondisk == "auto") {
+      if (nzc >= .Machine$integer.max) {
+        ondisk <- "yes"
+      } else
+        ondisk <- "no"
+    }
 
     if (!is_sparse(unwrapData(exprData, assay))) ## use sparse regime only
         sparse <- FALSE                          ## when input is sparse
@@ -230,20 +236,15 @@ gsvaParam <- function(exprData, geneSets,
         cli_alert_warning("Use it only if you are sure that such rows are not present in the input data")
     }
 
-    param <- new("gsvaParam",
-                 exprData=exprData, geneSets=geneSets,
-                 assay=assay, annotation=annotation,
-                 minSize=minSize, maxSize=maxSize,
-                 kcdf=kcdf, kcdfNoneMinSampleSize=kcdfNoneMinSampleSize,
-                 tau=as.double(tau), maxDiff=maxDiff, absRanking=absRanking,
-                 sparse=sparse, checkNA=checkNA, didCheckNA=naparam$didCheckNA,
-                 anyNA=naparam$any_na, use=use, filterRows=filterRows,
-                 nzcount=nzc, ondisk=ondisk)
-
-    maxmem <- .check_maxmem(param, "auto", verbose)
-    .check_ondisk(param, maxmem, verbose)
-
-    return(param)
+    new("gsvaParam",
+        exprData=exprData, geneSets=geneSets,
+        assay=assay, annotation=annotation,
+        minSize=minSize, maxSize=maxSize,
+        kcdf=kcdf, kcdfNoneMinSampleSize=kcdfNoneMinSampleSize,
+        tau=as.double(tau), maxDiff=maxDiff, absRanking=absRanking,
+        sparse=sparse, checkNA=checkNA, didCheckNA=naparam$didCheckNA,
+        anyNA=naparam$any_na, use=use, filterRows=filterRows, nzcount=nzc,
+        ondisk=ondisk)
 }
 
 
@@ -414,8 +415,8 @@ get_ondisk <- function(object) {
 #' @param recursive Not used with `x` being an object of
 #' class [`gsvaParam-class`].
 #'
-#' @aliases anyNA,gsvaParam-method
-#' @rdname gsvaParam-class
+#' @aliases anyNA,ssgseaParam-method
+#' @rdname ssgseaParam-class
 setMethod("anyNA", signature=c("gsvaParam"),
           function(x, recursive=FALSE)
             return(x@anyNA))
