@@ -28,6 +28,6 @@ plage.es[seq.int(min(nRowsToPrint, nrow(plage.es))),]
 c2BroadSets <- c2BroadSets[c(grep("_UP$", names(c2BroadSets)),
                              grep("_DN$", names(c2BroadSets)))]
 firPar <- plageParam(geneExpCostaEtAl2021, c2BroadSets,
-                     minSize=10, maxSize=500, verbose=FALSE)
+                     minSize=10, maxSize=500)
 fir_es <- gsva(firPar, verbose=FALSE)
 assay(fir_es)[seq.int(min(nRowsToPrint, nrow(fir_es))),]
